package com.itrycn.ta.script;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.media.AudioManager;
import android.telephony.TelephonyManager;
import android.util.DisplayMetrics;
import android.util.Size;

import com.itrycn.ta.AccessibilityOperator;

public class ScriptSystemInfo {
    private Context mContext;
    private static ScriptSystemInfo mInstance = new ScriptSystemInfo();

    private ScriptSystemInfo() {
    }

    public static ScriptSystemInfo getInstance() {
        return mInstance;
    }

    public void init(Context context) {
        mContext = context;
    }

    /**
     * 获取屏幕分辨率
     *
     * @return 返回分辨率
     */
    public Size GetScreen() {
        DisplayMetrics dm = new DisplayMetrics();
        dm = mContext.getResources().getDisplayMetrics();
        return new Size(dm.widthPixels, dm.heightPixels);
    }

    /**
     * 获取当前手机系统版本号
     *
     * @return 系统版本号
     */
    public String getSystemVersion() {
        return android.os.Build.VERSION.RELEASE;
    }

    /**
     * 获取手机型号
     *
     * @return 手机型号
     */
    public String getSystemModel() {
        return android.os.Build.MODEL;
    }

    /**
     * 获取手机厂商
     *
     * @return 手机厂商
     */
    public String getDeviceBrand() {
        return android.os.Build.BRAND;
    }

    public int getVersion(String packageName) {
        try {
            PackageManager manager = mContext.getPackageManager();
            PackageInfo info = manager.getPackageInfo(packageName, 0);
            return info.versionCode;
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }
    /**
     * 设置媒体音量
     *
     * @return
     */
    public void setMusicVolume(int value)
    {
        AudioManager audioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        audioManager.setStreamVolume(AudioManager.STREAM_MUSIC, value, AudioManager.FLAG_PLAY_SOUND);
    }
    /**
     * 获取媒体音量
     *
     * @return
     */
    public int getMusicVolume()
    {
        AudioManager audioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        return audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
    }
    /**
     * 设置闹钟音量
     *
     * @return
     */
    public void setAlarmVolume(int value)
    {
        AudioManager audioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        audioManager.setStreamVolume(AudioManager.STREAM_ALARM, value, AudioManager.FLAG_PLAY_SOUND);
    }
    /**
     * 获取闹钟音量
     *
     * @return
     */
    public int getAlarmVolume()
    {
        AudioManager audioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        return audioManager.getStreamVolume(AudioManager.STREAM_ALARM);
    }
    /**
     * 设置通知音量
     *
     * @return
     */
    public void setNotificationVolume(int value)
    {
        AudioManager audioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        audioManager.setStreamVolume(AudioManager.STREAM_NOTIFICATION, value, AudioManager.FLAG_PLAY_SOUND);
    }
    /**
     * 获取通知音量
     *
     * @return
     */
    public int getNotificationVolume()
    {
        AudioManager audioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        return audioManager.getStreamVolume(AudioManager.STREAM_NOTIFICATION);
    }
    /**
     * 设置电话铃声音量
     *
     * @return
     */
    public void setRingVolume(int value)
    {
        AudioManager audioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        audioManager.setStreamVolume(AudioManager.STREAM_RING, value, AudioManager.FLAG_PLAY_SOUND);
    }
    /**
     * 获取电话铃声音量
     *
     * @return
     */
    public int getRingVolume()
    {
        AudioManager audioManager = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        return audioManager.getStreamVolume(AudioManager.STREAM_RING);
    }
}
