package com.itrycn.tellmenotice;

import android.app.Notification;
import android.app.PendingIntent;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.graphics.Color;
import android.graphics.PixelFormat;
import android.provider.Settings;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import java.text.SimpleDateFormat;
import java.util.Date;

public class FloatView implements View.OnTouchListener {
    private Context mContext;
    private static WindowManager mWm;
    private WindowManager.LayoutParams mWinParams;
    private View view;
    private float mDownX;
    private float mDownY;
    private float mMoveX;
    private float mMoveY;

    public FloatView(Context context) {
        this.mContext = context;

        view = View.inflate(mContext, R.layout.float_tip, null);

        mWm = (WindowManager) context.getApplicationContext().getSystemService(
                Context.WINDOW_SERVICE);
        
        mWinParams = new WindowManager.LayoutParams();
        //mWinParams.height = WindowManager.LayoutParams.MATCH_PARENT;
        //mWinParams.width = WindowManager.LayoutParams.MATCH_PARENT;
        mWinParams.flags = WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE
                | WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON;
        mWinParams.format = PixelFormat.TRANSLUCENT;
        // mParams.windowAnimations =
        // com.android.internal.R.style.Animation_Toast;//动画设置
        //在小米手机的 5.0系统中，需要设置 type为 TYPE_TOAST，才能显示
        mWinParams.type = WindowManager.LayoutParams.TYPE_APPLICATION_OVERLAY;
        mWinParams.x = 0;
        mWinParams.y = 0;
        mWinParams.width = dip2px(mContext,250);
        mWinParams.height = dip2px(mContext,170);
        view.setOnTouchListener(this);
    }
    /**
     * 根据手机的分辨率从 dp 的单位 转成为 px(像素)
     */
    public static int dip2px(Context context, float dpValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dpValue * scale + 0.5f);
    }
    public void show() {
        mWm.addView(view, mWinParams);
    }
    public void showFloatingWindow(Notification notice, String Title, String Content) {
        showFloatingWindow(notice,Title,Content,-1);
    }
    public void showFloatingWindow(Notification notice, String Title, String Content, int TitleBackColor) {
        if (Settings.canDrawOverlays(mContext)) {
            LayoutInflater layoutInflater = LayoutInflater.from(mContext);
            ImageView image=view.findViewById(R.id.IdIcon);
            image.setImageIcon(notice.getSmallIcon());
            image.setOnClickListener(v -> {
                //打开通知栏的intent，即打开对应的聊天界面
                PendingIntent pendingIntent = notice.contentIntent;
                try {
                    pendingIntent.send();
                    if (mWm != null) mWm.removeView(view);
                } catch (PendingIntent.CanceledException e) {
                    Toast.makeText(mContext, "打开出错->"+e.getMessage(), Toast.LENGTH_SHORT).show();
                    e.printStackTrace();
                }
            });
            //image.setOnTouchListener(new FloatingOnTouchListener());
            ImageView image_close=view.findViewById(R.id.IdClose);
            image_close.setImageResource(R.drawable.close);
            image_close.setOnClickListener(v -> {
                if (mWm != null) mWm.removeView(view);
            });
            TextView idTitle = view.findViewById(R.id.IdTitle);
            if(TitleBackColor!=-1)
            {
                image.setBackgroundColor(TitleBackColor);
                idTitle.setBackgroundColor(TitleBackColor);
                image_close.setBackgroundColor(TitleBackColor);
            }
            //idTitle.setOnTouchListener(new FloatingOnTouchListener());
            TextView idContent = view.findViewById(R.id.IdContent);
            TextView idPostTime = view.findViewById(R.id.IdPostTime);
            idTitle.setText(Title);
            idContent.setText(Content);
            idContent.setOnClickListener(v -> {
                try {
                    ClipboardManager clipboard = (ClipboardManager)mContext.getSystemService(Context.CLIPBOARD_SERVICE);
                    clipboard.setPrimaryClip(ClipData.newPlainText(null,Content));
                } catch (Exception  e) {
                    Toast.makeText(mContext, "复制出错->"+e.getMessage(), Toast.LENGTH_SHORT).show();
                    e.printStackTrace();
                }
            });
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日 HH:mm:ss");// HH:mm:ss
            //获取当前时间
            Date date = new Date(System.currentTimeMillis());
            idPostTime.setText("时间:"+simpleDateFormat.format(date));
            //idContent.setOnTouchListener(new FloatingOnTouchListener());
            mWm.addView(view, mWinParams);
        }
    }
    public void hide(){
        if(mWm != null){
            mWm.removeView(view);
        }
    }

    @Override
    public boolean onTouch(View view, MotionEvent event) {

        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:

                mDownX = event.getRawX();
                mDownY = event.getRawY();

                break;
            case MotionEvent.ACTION_MOVE:

                /**
                 * 最为关键的就是拖动业务的坐标位置的计算
                 */

                mMoveX = event.getRawX();
                mMoveY = event.getRawY();

                float difX =  (mMoveX - mDownX);
                float difY =  (mMoveY - mDownY);

                mWinParams.x += difX;
                mWinParams.y += difY;
                mWm.updateViewLayout(view, mWinParams);

                mDownX = mMoveX;
                mDownY = mMoveY;

                break;
            case MotionEvent.ACTION_UP:

                break;

            default:
                break;
        }

        // 消费触摸事件
        return true;
    }

}
